//
//  TransMark.m
//  TransFormTest
//
//  Created by Amoson on 2016/10/16.
//  Copyright © 2016年 Amoson. All rights reserved.
//

#import "TransMark.h"
#import "AppDelegate.h"

@interface TransMark ()

//黑色背景视图
@property(nonatomic,strong)UIView * bgView;

//做动画白色view
@property(nonatomic,strong)UIView * animationbgView;

//动画类型
@property(nonatomic,assign)TransitionType tranType;

@end

@implementation TransMark

-(instancetype)initWithTransitionType:(TransitionType)type{
    self = [super init];
    if (self) {
        self.tranType = type;
    }
    return self;
}


- (NSTimeInterval)transitionDuration:(nullable id <UIViewControllerContextTransitioning>)transitionContext{
    
    return 0.75;
}

- (void)animateTransition:(id <UIViewControllerContextTransitioning>)transitionContext{
    
    switch (self.tranType) {
        case TransitionTypePush:
        {
            //push
            [self pushTransitionAnimationWithContext:transitionContext];
        }
        break;
        case TransitionTypePop:
        {
            //pop
            [self popTransitionAnimationWithContext:transitionContext];

        }
        break;
      
    }
    
}

-(void)pushTransitionAnimationWithContext:(id <UIViewControllerContextTransitioning>)transitionContext{
    /*获取fromVc toVc  容器containerView*/
    UIViewController *  fromVc = [transitionContext viewControllerForKey:UITransitionContextFromViewControllerKey];
    UIViewController * toVc = [transitionContext viewControllerForKey:UITransitionContextToViewControllerKey];
    UIView * containerView= [transitionContext containerView];

    for (UIView * someView in containerView.subviews) {
        if (someView) {
            [someView removeFromSuperview];
        }
    }
    /* 通过kvo获取UITableView */
    UITableView * tableView = (UITableView*)[fromVc valueForKeyPath:@"tableView"];
    
    /* 获取cell在父视图上的坐标转换 */
    CGRect rectInTableView = [tableView rectForRowAtIndexPath:[tableView indexPathsForSelectedRows].firstObject];
    CGRect rect = [tableView convertRect:rectInTableView toView:[tableView superview]];
    
    /* 赋值给做动画的bgview */
    self.animationbgView.frame = rect;
    
    /* 截取UIWindow的view */
    AppDelegate * delegate = (AppDelegate*)[UIApplication sharedApplication].delegate;
    UIWindow * window = delegate.window;
    UIView * sanpView = [window snapshotViewAfterScreenUpdates:NO];
    sanpView.frame = [UIScreen mainScreen].bounds;
    
    /* 设置toVc的frame */
    toVc.view.frame = [transitionContext finalFrameForViewController:toVc];
    toVc.view.alpha = 0;
    
    /* 添加view到当前容器 注意 ：toVc.view 应该在最前面 */
    
    [containerView addSubview:toVc.view];
    [containerView addSubview:self.bgView];
    [containerView addSubview:sanpView];
    [containerView addSubview:self.animationbgView];
    
    [UIView animateWithDuration:0.3 animations:^{
        [containerView layoutIfNeeded];
        
        /* 隐藏导航栏 */
        toVc.navigationController.navigationBar.hidden = YES;
        toVc.navigationController.navigationBar.alpha = 0;
        
        /* 缩放截屏view */
        sanpView.transform = CGAffineTransformMakeScale(0.96, 0.96);
    } completion:^(BOOL finished) {
        [UIView animateWithDuration:0.3 animations:^{
            /* 白色bgview动画 */
            self.animationbgView.frame = [UIScreen mainScreen].bounds;
            
        } completion:^(BOOL finished) {
            sanpView.hidden = YES;
            self.bgView.hidden = YES;
            [self.animationbgView removeFromSuperview];
            
            [UIView animateWithDuration:0.15 animations:^{
                
                toVc.navigationController.navigationBar.hidden = NO;
                toVc.navigationController.navigationBar.alpha = 1;
                toVc.view.alpha = 1;
                
            }];
        }];
        //告诉系统动画结束
        [transitionContext completeTransition:!transitionContext.transitionWasCancelled];
    }];

}

-(void)popTransitionAnimationWithContext:(id <UIViewControllerContextTransitioning>)transitionContext{
    /*获取fromVc toVc  容器containerView*/
    UIViewController *  fromVc = [transitionContext viewControllerForKey:UITransitionContextFromViewControllerKey];
    UIViewController * toVc = [transitionContext viewControllerForKey:UITransitionContextToViewControllerKey];
    UIView *containerView = [transitionContext containerView];
    
    [containerView insertSubview:toVc.view atIndex:0];
    
    UIView * tempView = containerView.subviews.lastObject;
    UIView * bgView = containerView.subviews[1];
    bgView.backgroundColor = [UIColor blackColor];
       /* 隐藏导航栏 */
    toVc.navigationController.navigationBar.hidden = YES;
    fromVc.view.hidden = YES;
    bgView.hidden = NO;
    tempView.hidden = NO;
    toVc.view.hidden = YES;
    
    [UIView animateWithDuration:[self transitionDuration:transitionContext]/2 animations:^{
        tempView.transform = CGAffineTransformIdentity;
    } completion:^(BOOL finished) {
        [toVc.navigationController.navigationBar setHidden:NO];
        fromVc.view.hidden = NO;
        toVc.view.hidden = NO;
        
        if ([transitionContext transitionWasCancelled]) {
            //失败了标记失败
            [transitionContext completeTransition:NO];
        }else{
            //如果成功了，我们需要标记成功，同时让vc1显示出来，然后移除截图视图，
            [transitionContext completeTransition:YES];
            [tempView removeFromSuperview];
            [bgView removeFromSuperview];
        }
    }];
}

/*
 * 懒加载黑色背景view
 */
-(UIView*)bgView{
    if (!_bgView) {
        _bgView = [UIView new];
        _bgView.backgroundColor = [UIColor blackColor];
        _bgView.frame = [UIScreen mainScreen].bounds;
    }
    return _bgView;
}

/*
 * 懒加载白色动画view
 */
-(UIView*)animationbgView{
    if (!_animationbgView) {
        _animationbgView = [UIView new];
        _animationbgView.backgroundColor = [UIColor whiteColor];
    }
    return _animationbgView;
}


@end
